# code_R_bmo
# zuletzt bearbeitet von frauke
# am 221012

# in diesem Code-Stueck wird die bmo erstellt - die Kraftwerke in R eingelesen und Grenzkosten entsprechend type und fuel des Kraftwerks zugeordnet und je Region nach Grenzkosten geordnet

#----------------------------------
# Kraftwerke aus Datenbank einlesen

bmo_env <- function(){
  
sql_pp 		<- paste("SELECT pp_nr, pinst, dpr, fuel, type, year FROM `pp` WHERE (`status` = 'opr' OR (`status` = 'con' AND `year` < '",scenario_year,"')) AND (retire = 0 OR retire >= '",scenario_year,"') AND (dpr >= '",min_reg,"' AND dpr <= '",max_reg,"')", sep="")

pp_rawdata 		<- dbGetQuery(con_baltic, sql_pp) 

# das sind alle Kraftwerke in Operation. Dazu kommen noch die Kraftwerke mit Status con (under construction) oder pln (planned) die ein Beginn YEAR angegeben haben das unter dem Szenario-Jahr liegt.

#---------------------------------
# Krafwerke in einem data.frame pp_data zusammenfuehren
pp_nr 	<- c(as.numeric(as.character(pp_rawdata$pp_nr)))
type 	<- c(as.character(pp_rawdata$type))
pinst 	<- c(as.numeric(as.character(pp_rawdata$pinst)))
reg 	<- c(as.numeric(as.character(pp_rawdata$dpr)))
fuel 	<- c(as.character(pp_rawdata$fuel))
age 	<- c(as.numeric(as.character(pp_rawdata$year)))

fuel[which(fuel == "bfg")] <- "gas"

# data.frame mit allen weiter benoetigten Infos der Kraftwerke
pp_data <- data.frame(pp_nr,type,pinst,reg,fuel,age) 

#------------------------------------------------------
# bei welchen Kraftwerken ist die lifetime abgelaufen?
#------------------------------------------------------
# retired Kraftwerke aus dem pp_data data.frame rausschmeissen. Je nach gewaehlter liftime (in code_R_chose) - unterschiedlich je FUEL gibt es ein Retire-Jahr - das ist dann ein Vektor und alle deren Retire-Jahr vor dem Szenario-Jahr liegt kommen raus.

fuel_lifetime 			<- c("lignite","hard_coal","gas","oil","refuse")
lifetime 			<- c(lifetime_lig, lifetime_coal, lifetime_gas, lifetime_oil, lifetime_ref)
idx_lifetime 			<- match(pp_data$fuel, fuel_lifetime) 
idx_lifetime_NA 		<- which(is.na(idx_lifetime) == TRUE) # hier sind noch NA dabei, da ich nicht fuer alle fuel-typen mit fuel_lifetime ein alter ausgewaehlt habe. 
idx_lifetime[idx_lifetime_NA] 	<- 3 # Daher bekommen alle uebrigen Typen die lifetime 40 Jahre
pp_data$age[age == 0] 		<- 1990 # wenn kein Jahr angegeben ist wird einfach 1990 genommen
pp_data$age[which(is.na(pp_data$age) == TRUE)] <- 1990 # wenn kein Jahr angegeben ist wird einfach 1990 genommen
retire 				<- lifetime[idx_lifetime] + pp_data$age # Berechnung des Retire-Jahres der Kraftwerke
idx_retire 			<- which(retire < scenario_year) # Index-Vektor der im zu berechnenden Szenario bereits vom Netz genommenen Kraftwerke

lifetime_pp     <- lifetime[idx_lifetime]
lifetime_pp     <- lifetime_pp[-idx_retire]

# die im scenario_year bereits retireten Kraftwerke werden allesamt aus dem pp_data rausgenommen. 
pp_data 			<- pp_data[-idx_retire,]

#-------------------------------------------------------------------------------------------------------------
# Daten fuer Cmar-Berechnung zusammensuchen -- erst allgemein dann fuer jedes Kraftwerk nach dem type zuordnen
#-------------------------------------------------------------------------------------------------------------
# Die parameter zur Cmar Berechnung einlesen
pp_parameter 		<- dbGetQuery(con_baltic, "SELECT Cvar, effbrutto, aux, fuel, type, Cfix, Cinv FROM pp_parameter")

# die fuel-Preise abhaengig vom ausgewaehlten scenario_year und fuel_price_scenario auslesen
sql_fuel_price 		<- paste("SELECT fuel,pr_fuel FROM pp_fuel_price WHERE scenario = '",fuel_price_scenario,"' AND year_fuel = '",scenario_year,"' ", sep = "")
fuel_price 	 	<- dbGetQuery(con_baltic, sql_fuel_price)
idx_fuel_price 		<- match(pp_parameter$fuel, fuel_price$fuel) # Indexvektor fuer die Reihenfolge in der sich die Preise fuer die unterschiedlichen Brennstoffe in die pp_parameter-Tabelle ordnen sollen

# der emissionspreis abhaengig vom ausgewaehlten scenario_year und em_price_scenario auslesen
sql_em_price 		<- paste("SELECT PRem FROM pp_emission WHERE scenario = '",em_price_scenario,"' AND year_emission = '",scenario_year,"' ", sep = "")
em_price 	 	<- as.numeric(as.character(dbGetQuery(con_baltic, sql_em_price)))

# die emissionsfaktoren in Abhaengigkeit vom Brennstoff
fuel_em 	 	<- dbGetQuery(con_baltic, "SELECT fuel, em_fuel FROM pp_fuel_em")
idx_fuel_em 		<- match(pp_parameter$fuel, fuel_em$fuel) # Indexvektor fuer die Reihenfolge in der sich die Emissionsfaktoren fuer die unterschiedlichen Brennstoffe in die pp_parameter-Tabelle ordnen sollen

pp_parameter 		<- cbind(pp_parameter[,c(1,2,3,4,5,6,7)],fuel_price$pr_fuel[idx_fuel_price],fuel_em$em_fuel[idx_fuel_em])
colnames(pp_parameter) 	<- c("Cvar","effbrutto","aux","fuel","type","Cfix","Cinv","fuel_price","em_rate")

#------------------------------
# pp_parameter pp_data zuordnen
#------------------------------
# jedes Kraftwerk bekommt einzeln seine Parameter, denn die Cmar wird fuer jedes einzelne Kraftwerk berechnet
fuel_type_parameter 	<- paste(pp_parameter$fuel,pp_parameter$type, sep = "_")
fuel_type_pp 		<- paste(pp_data$fuel,pp_data$type, sep = "_")
idx_type 		<- match(fuel_type_pp, fuel_type_parameter) # Indexvektor je Kraftwerk definiert ueber type 
pp_data_parameter 	<- pp_parameter[idx_type,]

#----------------------------------------------
# effbrutto in Abhaengigkeit vom Alter anpassen
#----------------------------------------------
# Wirkungsgrad wird dem Alter angepasst, das passiert jetzt fuer jedes einzelne Kraftwerk
# das 0.003 koennte ich nochmal je nach Technologie dann anpassen
effbrutto_age 	<- pp_data_parameter$effbrutto - (1980 - pp_data$age) * 0.003

#-----------------
# Brennstoffkosten
#-----------------
Cfuel 		<- ifelse((pp_data_parameter$effbrutto) == 0,0,1/(effbrutto_age - pp_data_parameter$aux) * 3.6 * pp_data_parameter$fuel_price)
# falls effbrutto und aux 0 sind, ist Cfuel auch null, denn dann sind die Cmar schon vorab in der Tabelle festgelegt ist, das ist bei den Atomkraftwerken der Fall

#----------------
# Emissionskosten
#----------------
co2 		<- ifelse((pp_data_parameter$effbrutto - pp_data_parameter$aux) == 0,0,(1/(effbrutto_age - pp_data_parameter$aux) * 3.6 * pp_data_parameter$em_rate))
Cem 		<- co2 * em_price

#----------------
# Fixkosten & Investitionskostem
#----------------
Cfix    <- pp_data_parameter$Cfix
Cinv    <- pp_data_parameter$Cinv

# -----------------
# Grenzkosten Cmar
#------------------

Cmar 		<- pp_data_parameter$Cvar + Cfuel + Cem

fuel 		<- as.character(pp_data$fuel)
fuel 		<- ifelse(fuel %in% c("uran","lignite","oil","gas","hard_coal","refuse"),fuel,"OTHER")
pp_data$fuel 	<- fuel
pp_data 	<- cbind(pp_data,Cmar,co2) # der Cmar Vektor und co2 pro prod. kWhwird noch in pp_data integriert

output <- list(pp_data = pp_data, Cfix = Cfix, Cinv = Cinv, lifetime_pp = lifetime_pp)

return(output)
}

bmo_results<- bmo_env()
pp_data    <- bmo_results$pp_data
Cfix       <- bmo_results$Cfix
Cinv       <- bmo_results$Cinv
lifetime_pp <- bmo_results$lifetime_pp

#--------------------------------------------------
# Der pp_data data.frame ist die Vorlage fuer die bmo, fuer jede Region soll es nun einen Abschnitt geben der nach Cmar innerhalb der Region sortiert ist.
#--------------------------------------------------
cap_fossil          <- sum(pp_data$pinst)
cap_fossil          <- ifelse (is.na(cap_fossil), 0, cap_fossil)

# -----------------
# Parameter der Wasserkraftwerke
# -----------------
Cvar_HY                 <- dbGetQuery (con_baltic, "SELECT Cvar, Cfix, Cinv FROM pp_parameter WHERE fuel = 'hydro'")
Cfix_HY                 <- as.numeric(as.character(Cvar_HY$Cfix))
Cinv_HY                 <- as.numeric(as.character(Cvar_HY$Cinv))
Cvar_HY                 <- as.numeric(as.character(Cvar_HY$Cvar))

# Diese Parameter werden fuer die Wirtschaftlichkeitsbetrachtung gebraucht.

if("fos_inst" %in% ls() && fos_inst != 0){source("code_R_bmo_cut.R")}
