# code_R_demand
# zuletzt bearbeitet von frauke 
# 051012

# In diesem Code-Stueck wird die Demand vorbereitet - Basis ist das Nachfragejahr, aus der Datenbank wird die entsprechende Stunden-Zeitreihe fuer alle Laender ausgelesen und entsprechend dem gewaehlten demand_change werden alle Nachfragekurven der Laender skaliert. Momentan kann man den demand_change nur fuer alle Laender gesamt auswaehlen, das waere aber leicht anpassbar.

#--------------------------------

demand_sql 		<- paste("SELECT ",time_unit," , country_id, demand FROM demand_",time_unit," WHERE year = '2011' AND ",time_unit," <= ",end_timestep,"", sep = "")
demand_data 			<- dbGetQuery(con_baltic, demand_sql)
demand_data 			<- demand_data[order(demand_data$country_id),] # nach der Laender-Nummer sortieren

#----------------------------
# Regionen auswaehlen
demand_de <- subset(demand_data, demand_data$country_id == 1)

if (area == "de")     {demand <- demand_de}
if (area == "no_de")  {demand <- subset(demand_data, demand_data$country_id %in% c(1,9))}
if (area == "baltic") {demand <- demand_data}

demand_reg  <- matrix(demand$demand, ncol = region_abroad + 1, nrow = end_timestep) # Nachfrage in Matrixform

#----------------------------
if (region_split_de > 1) {
  # Nachfrage in Deutschland aufteilen - nur falls region_split_de > 1
  # In der Datenbank sind die Prozentzahlen fuer die vier DENA-Zustaende und die Maximallasst 2020 je Region. Als Standard waehle ich den Mittelwert aus den 4 DENA-Zahlen von 2013
  distribution  <- dbGetQuery(con_baltic, "SELECT portion_demand_de FROM demand_distribution WHERE situation != 'maximal_demand_2020'")
  distribution 	<- matrix(c(as.numeric(as.character(unlist(distribution)))), ncol = 4) # in Matrixform
  distribution 	<- rowMeans(distribution) # die Zeilendurchschnitte, das ist die Zahl die den Anteil jeder dpr an der demand angibt

  demand_de_dpr <- demand_de$demand %*% t(distribution) # Die Werte fuer die demand_de (Nachfrage in Deutschland) werden mit den 21 Werten fuer distribution multipliziert und zwar jeder mit jedem, so dass demand_dpr dann eine Matrix mit soviel Zeilen wie Zeitschritte und 21 Spalten ist

  demand_reg_de <- matrix(NA, ncol = region_split_de, nrow = timesteps)

  
  if (region_split < 21){
    # Wenn Deutschland in weniger als 21 Regionen aufgeteilt wird, werden die Regionen wieder zusammen gefasst.
    # die Aufteilung der dpr auf die Regionen ist in region_list gespeichert
    for(reg in 1:region_split_de){
      r <- region_list[[reg]]
      if(length(r) == 1){
        demand_reg_de[,reg] <- demand_de_dpr[,(region_list[[reg]]-100)]
      }else{
        demand_reg_de[,reg] <- rowSums(demand_de_dpr[,(region_list[[reg]]-100)])
      }
    }
  } else {
    demand_reg_de <- demand_de_dpr # Ansonsten werden die dispatch-Regionen direkt uebernommen.
  }
  
  if (area == "de"){
    demand_reg <- demand_reg_de
  } else {
  demand_reg <- cbind(demand_reg_de, demand_reg[,2:ncol(demand_reg)]) # Die deutschen Regionen werden an die internationale Nachfrage gehaengt.
}
}

#----------------------------

# Variation der Nachfrage gemaess gewaehlten demand_scenario
demand_scenario_sql 	<- paste("SELECT country_id, demand_change FROM demand_scenario WHERE scenario_name = '",demand_scenario,"'", sep = "")
demand_sc 		<- dbGetQuery(con_scenarios, demand_scenario_sql)
demand_sc 		<- demand_sc[order(demand_sc$country_id),]
demand_sc     <- subset(demand_sc, demand_sc$country_id %in% country_list)

#-----------------------------
# den demand_change auf das scenario_year runterbrechen
demand_factor 		<- (scenario_year - 2010) / (2050 - 2010)

# Nachfragekurve Laender skalieren entsprechend dem gewaehlten demand_change - bei allen Laendern gleiche variation
variation 		    <- (demand_sc$demand_change * demand_factor) + 1
variation_de      <- rep(variation[1], times = region_split_de) # Die Veraenderung in Deutschland ist in allen Regionen gleich.

if (area != "de") {
  variation       <- c(variation_de, variation[2:length(variation)])
} else {
  variation       <- variation_de
}

variation_matrix 	<- matrix(rep(variation, end_timestep), ncol = region_split, byrow = TRUE)

demand_reg 		    <- demand_reg * variation_matrix # die demand mit der gewaehlten Aenderungsrate multipliziert
