# code_R_economics
# zuletzt bearbeitet von gesine
# 131112

# In diesem Code-Stuekc werden aufbauend auf den Ergebnissen des Szenarios Wirtschaftliche Indikatoren und Ergebnisse berechnet

# ------------------------------------------------------------
# Wirtschaftlichkeit von Kraftwerken

# -----------------
# Volllaststunden / Ausnutzung von Kraftwerken, Pumpen
# -----------------

# Fuer jedes Kraftwerk wird die gesamte produzierte Menge aus den Matrizen elp und elp_pump bestimmt

elp_total         <- colSums(elp*energy_factor) # Die MW pr quarter werden in MWh umgerechnet
pinst_pump        <- c(pump$pump_pinst, stor_pot_oth)

flh               <- elp_total/unit_standard_data$pinst # Volllaststunden

elp_total_pump    <- colSums(elp_pump_each * energy_factor)

flh_pump          <- elp_total_pump/pinst_pump

# -----------------
# Umsatz
# -----------------

# Fuer den Umsatz wird die produzierte Menge in jeder Viertelstunde mit dem Preis multipliziert

revenue                <- numeric(length(unit_standard))
expenditure_pump       <- numeric(length(pump_standard))

for (r in 1:region_split) {
  pp_part 		          <- subset(unit_standard_data, reg %in% region_list[[r]])
  idx_unit_standard     <- match(pp_part$pp_nr, unit_standard)
  revenue_pp_quarter  <- matrix(ncol=length(idx_unit_standard), nrow=timesteps)
  
  pump_region           <- subset(pump_data, pump_data$reg_pump %in% region_list[[r]])
  idx_pump_standard     <- match(pump_data$pump_nr, pump_standard)
  expenditure_pump_quarter  <- matrix(ncol=length(idx_pump_standard), nrow=timesteps)

  for (i in start_timestep:end_timestep){
  revenue_pp_quarter[i,]    <- elp[i,idx_unit_standard] * energy_factor * price_exchange[i,r]
  expenditure_pump_quarter[i,]  <- elp_pump_each[i,idx_pump_standard]* energy_factor * price_exchange[i,r]
  }
  
  revenue_pp                  <- colSums(revenue_pp_quarter) / 1000000
  revenue[idx_unit_standard]  <- revenue_pp           # in Millionen Euro
  
  expenditure_one_pump    <- colSums(expenditure_pump_quarter) / 1000000
  expenditure_pump[idx_pump_standard] <- expenditure_one_pump  # in Millionen Euro
}

expenditure_pump          <- - expenditure_pump # bei den Pumpen sind das die Ausgaben fuer Strom, also negativ

# -----------------
# Wirtschaftlichkeit
# -----------------

# Deckungsbeitrag - Erloese minus variable Kosten
# Cmar aendert sich bei den Wasserkraftwerken mit dem Fuellstand. Das sind aber Opportunitaetskosten, 
# daher werden hier die festen Kosten Cvar zu Grunde gelegt

Cvar_turb               <- rep.int(Cvar_HY, times=length(turb$turb_nr))
Cmar_bio                <- rep.int(40, times = length(biomass_power))
Cmar_stor_oth           <- rep.int(50, times = length(stor_pot_oth))
Cmar_all                <- c(pp_data$Cmar, Cvar_turb, Cmar_bio, Cmar_stor_oth) # Kosten fuer alle Kraftwerke

cover                   <- revenue - Cmar_all * elp_total / 1000000  # Deckungsbeitrag (coverage) fuer alle Kraftwerke, in Million Euro

# Wird den fixen Kosten Cfix und Cinv gegenueber gestellt
#################################
# investitionskosten fuer die anderen Technologien
#################################

Cfix_turb               <- rep.int(Cfix_HY, times=length(turb$turb_nr))
Cfix_bio                <- rep.int(20, times = length(biomass_power))
Cfix_stor_oth           <- rep.int(20, times = length(stor_pot_oth))
Cfix_all                <- c(Cfix, Cfix_turb, Cfix_bio, Cfix_stor_oth)
cover_Cfix              <- cover - Cfix_all * unit_standard_data$pinst / 1000 # Euro/kW * MW * 1000 / 1000000 -> Million Euro

Cinv_turb               <- rep.int(Cinv_HY, times=length(turb$turb_nr)) # Investitionskosten in EURO/kW
Cinv_bio                <- rep.int(1000, times = length(biomass_power))
Cinv_stor_oth           <- rep.int(1500, times = length(stor_pot_oth))
Cinv_all                <- c(Cinv, Cinv_turb, Cinv_bio, Cinv_stor_oth)
lifetime_turb           <- rep.int(45, length(turb$turb_nr))
n_bio                   <- rep.int(35, length(biomass_power))
n_stor_oth              <- rep.int(45, length(stor_pot_oth))
n_pp                    <- c(lifetime_pp, lifetime_turb, n_bio, n_stor_oth)
annu                    <- Cinv_all * unit_standard_data$pinst * 1000 * (interest_rate * (1 + interest_rate)^n_pp)/((1+interest_rate)^n_pp - 1) # Kreditannuitaet
cover_Cinv              <- cover_Cfix - annu / 1000000 # Deckungsbeitrag zur Annuitaet, million Euro                               

# Capital Value
capital_value <- - Cinv_all + revenue * ((1 + interest_rate)^n_pp - 1) / ((1 + interest_rate)^n_pp * interest_rate)
       
# -----------------
# Daten in die Datenbank schreiben
# -----------------

# Die Indikatoren der Wirtschaftlichkeit f?r jedes Kraftwerk werden in einem data.frame zusammengefasst

pp_economics              <- data.frame(cbind(scenario_nr, unit_standard, flh, elp_total, revenue, cover, cover_Cfix, cover_Cinv, capital_value))
colnames(pp_economics)    <- c("scenario_nr", "pp", "production", "flh", "revenue", "coverage", "cfix_coverage", "cinv_coverage", "capital_value")

pump_economics            <- data.frame(cbind(scenario_nr, pump_standard, elp_total_pump, flh_pump, expenditure_pump))
colnames(pump_economics)  <- c("scenario_nr", "pump", "pumped electricity", "flh", "expenditure")

dbWriteTable (con_results, "economics_pp", pp_economics, row.names = FALSE, overwrite = FALSE, append = TRUE) 
dbWriteTable (con_results, "economics_pump", pump_economics, row.names = FALSE, overwrite = FALSE, append = TRUE) 

# Gesammelte Daten von Pumpspeicherkraftwerken
sql_pump_storage_1 <- paste("INSERT INTO results_baltic.economics_pumped_storage 
SELECT renpass_baltic.storage.stor_location, economics_pp.scenario_nr, sum(results_baltic.economics_pp.production), sum(results_baltic.economics_pump.pumped_electricity), sum(results_baltic.economics_pp.revenue) 
FROM  renpass_baltic.storage 
LEFT OUTER JOIN results_baltic.`economics_pp`
ON  economics_pp.pp = storage.stor_nr
AND economics_pp.scenario_nr = ",scenario_nr," 
LEFT OUTER JOIN results_baltic.economics_pump
ON economics_pump.pump = storage.stor_nr 
AND economics_pump.scenario_nr = ",scenario_nr," GROUP BY storage.stor_location", sep = "")
dbSendQuery(con_results, sql_pump_storage_1)
sql_pump_storage_2 <- paste("UPDATE economics_pumped_storage SET scenario_nr = ",scenario_nr," WHERE scenario_nr = 0", sep = "")
dbSendQuery(con_results, sql_pump_storage_2)


# ---------------------------------------------------
# Wirtschaftlichkeit von Netzen

# ----------------
# Ausnutzung und Wirtschaftlichkeit
# ----------------

number_lines                  <- length(grid_exist[,1])

exchange_cable                <- numeric(length = number_lines) # in MWh
flh_cable                     <- numeric(length = number_lines)
grid_charge_cable             <- numeric(length = number_lines)
revenue_cable               <- numeric(length = number_lines)
revenue_cable_quarter       <- matrix(0, nrow = number_lines, ncol = timesteps)

idx_pos             <- which(exchange_pump_combined[,3] >= 0)
idx_neg             <- which(exchange_pump_combined[,3] < 0)
exchange_pos             <- numeric(length = number_lines)
exchange_neg             <- numeric(length = number_lines)

n_grid                        <- 30 # lebensdauer des netzes fuer die annuitaet
Cinv_grid                     <- 200 # Investitionskosten Netz euro pro kWh und Leitung

for (c1 in 1:number_lines) {
    
    idx_cable                 <- seq(c1, number_lines * timesteps, by = number_lines)
    idx_from_reg              <- which(regions == grid_exist[c1,1])
    idx_to_reg                <- which(regions == grid_exist[c1,2])
    exchange_cable[c1]        <- sum(abs(exchange_pump_combined[idx_cable,3] * energy_factor))
    idx_cable_pos             <- intersect(idx_pos, idx_cable)
    idx_cable_neg             <- intersect(idx_neg, idx_cable)
    
    exchange_pos[c1]          <- sum(exchange_pump_combined[idx_cable_pos,3]) * energy_factor
    exchange_neg[c1]          <- sum(exchange_pump_combined[idx_cable_neg,3]) * energy_factor
    
    flh_cable[c1]             <- exchange_cable[c1] / (grid_exist[c1, 3])
    
# Merchant Cable - Congestion Rent
    revenue_cable_quarter[c1,]   <- exchange_combined[idx_cable,3]*energy_factor * (price_exchange[,idx_to_reg] - price_exchange[,idx_from_reg]) / 1000000 # in Million Euro
    
    revenue_cable_quarter[c1,]  <- ifelse(revenue_cable_quarter[c1,] < 0, 0, revenue_cable_quarter[c1,])
    
    revenue_cable[c1]           <- sum(revenue_cable_quarter[c1,])

}

# Annuitaet der Investition
    
annu_grid                 <- Cinv_grid * grid_exist[,3] * 1000 * (interest_rate * (1 + interest_rate)^n_grid)/((1+interest_rate)^n_grid - 1)
cover_inv_grid_merch      <- revenue_cable - annu_grid

# Capital Value
capital_value_grid        <- - Cinv_grid * grid_exist[,3] + revenue_cable * ((1 + interest_rate)^n_grid - 1) / ((1 + interest_rate)^n_grid * interest_rate)


grid_economics            <- data.frame(cbind(scenario_nr, grid_exist[,1], grid_exist[,2], grid_exist[,3], exchange_cable, exchange_pos, exchange_neg, flh_cable, revenue_cable, cover_inv_grid_merch, capital_value_grid))
colnames(grid_economics)  <- c("scenario_nr", "plus_region", "minus_Region", "grid_capacity", "exchange_total", "exchange_positive", "exchange_negative", "flh", "revenue",  "coverage", "capital_value")

dbWriteTable (con_results, "economics_grid", grid_economics, row.names = FALSE, overwrite = FALSE, append = TRUE) 

