# Name: code_R_filling_level
# geaendert von: gesine
# am 260713

# Hier wird die maximale angebotene Menge der Pumpen und Turbinen aus dem Fuellstand der Reservoire und der verfuegbaren Speicherkapazitaet berechnet.
# Dazu werden die Fuellstaende aus der fil Matrix in Energie umgerechnet und jeweils die oberen und unteren Reservoire fuer Turbinen und Pumpen zugeordnet.

# Input: fill (Fuellstand der Reservoire), Verbindung Kraftwerk-Reservoir, Pinst der Pumpen, Turbinen , maximaler Fuellstand der Reservoire

# -----   Output
# bid_turb        - angebotene Menge der Turbinen [MW] 
# bid_pump        - maximal nachgefragte Menge der Pumpen [MW]
# share_pump      - relativer Fuellstand der oberen Reservoirs der Pumpen 
# share_turb      - relativer Fuellstand der oberen Reservoire der Wasserkraftwerke                                 
                                  
#------- Ablauf

# Energieinhalt der oberen Reservoire der Turbinen
# verfuegbarer Speicherplatz in den oberen Reservoire der Pumpen
# verfuegbarer Speicherplatz in den unteren Reservoiren der Turbinen
# Energieinhalt der unteren Reservoire der Pumpen
# Maximales Angebot bestimmen
# Gebote werden nach unten bei Null begrenzt
# Aus dem relativen Fuellstand bzw. dem prognostizierten Ueberlauf wird der Preis bestimmt
# Das data.frame aus Name, Gebot, Grenzkosten und dpr wird fuer die Turbinen erstellt

#---------------------------

# Die Kapazitaet (fill_max) und der Fuellstand (fil) der Reservoire ist in der fil Matrix in mio cbm angegeben und wird fuer die Bestimmung des Angebots in MW umgerechnet.
# Dazu werden die Daten aus den Verknuepfungstabellen und Indizes verwendet, die in prep_storage definiert wurden.

# Verknuepfung zu oberen Reservoiren

fil_up       <- fil[i, idx_upper_res] # Fuellstaende in der Laenge und Reihenfolge von turb_res_upper

fil_forecast <- fil[i,] + sum(flo[i:(min(end_timestep, (i+(168/energy_factor))))])*res_flo

fil_spil     <- fil_forecast - fil_max

flo_forecast <- sum(flo[i:(min(end_timestep, (i+(12/energy_factor))))])*res_flo

fil_spil_up <- fil_spil[idx_upper_res]

# Die Operationen fuer Turbinen und Pumpen werden jeweils fuer alle Anlagen durchgefuehrt, es wird erst hinterher wieder getrennt.
# Das liegt daran, dass die Daten aus fil erst pro Turbine / Pumpe zusammengefasst werden muessen, um eindeutig zugeordnet werden koennen.

fil_turb_up         <- fil_up * turb_per_res_upper * efactor_upper * 1000  / energy_factor 
fil_spil_turb_up    <- fil_spil_up * turb_per_res_upper * efactor_upper * 1000  / energy_factor
fil_pump_up         <- (fil_max_up - fil_up) * pump_per_res_upper * efactor_upper * 1000 / energy_factor

flo_forecast_up <- flo_forecast[idx_upper_res] * turb_per_res_upper * efactor_upper * 1000  / energy_factor

# Fuellstaande werden pro Turbine summiert
# als Energieinhalt
sum_turb_up         <- aggregate(fil_turb_up, list(stor_upper), sum) # nach dem aggregate Befehl stehen die Werte in der 2. Spalte 
sum_spil_turb_up    <- aggregate(fil_spil_turb_up, list(stor_upper), sum)
sum_pump_up         <- aggregate(fil_pump_up, list(stor_upper), sum)

sum_flo_forecast_up <- aggregate(flo_forecast_up, list(stor_upper), sum)

# Die Fuellstaande in mio cbm werden aufsummiert, um den relativen Fuellstand zu berechnen.
sum_fil_up          <- aggregate(fil_up, list(stor_upper), sum) 
share_up            <- sum_fil_up[,2]/sum_fil_max_up[,2]

share_up_turb       <- share_up[idx_turb_up]

up_turb             <- sum_turb_up[idx_turb_up, 2]
up_pump             <- sum_pump_up[idx_pump_up, 2]

up_spil <- sum_spil_turb_up[idx_turb_up,2]
share_pump_up       <- share_up[idx_pump_up]

share_flo_up        <- sum_flo_forecast_up[,2]/sum_fil_max_up[,2]
share_flo_up_turb   <- share_flo_up[idx_turb_up]

# Verknuepfung mit den unteren Reservoiren, hier gibt es nicht fuer alles Wasserkraftwerke eine Verbindung
fil_lo              <- fil[i, idx_lower_res]     # Fuellstaende in der Laenge und Reihenfolge von turb_res_upper
fil_spil_lo <- fil_spil[idx_lower_res]

fil_turb_lo         <- (fil_max_lo - fil_lo) * turb_per_res_lower * efactor_lower * 1000 / energy_factor
fil_spil_turb_lo    <- fil_spil_lo * turb_per_res_lower * efactor_lower * 1000 / energy_factor
fil_pump_lo         <- fil_lo * pump_per_res_lower * efactor_lower * 1000 / energy_factor

sum_turb_lo         <- aggregate(fil_turb_lo, list(stor_lower), sum) # Fuellstaande werden pro Turbine summiert
sum_spil_turb_lo    <- aggregate(fil_spil_turb_lo, list(stor_lower), sum)
sum_pump_lo         <- aggregate(fil_pump_lo, list(stor_lower), sum)

# Die Fuellstaande in mio cbm werden aufsummiert, um den relativen Fuellstand zu berechnen.
sum_fil_lo          <- aggregate(fil_lo, list(stor_lower), sum) 
share_lo            <- sum_fil_lo[,2]/sum_fil_max_lo[,2]
share_lo_turb       <- share_lo[idx_turb_lo]
share_lo_turb       <- ifelse(is.na(share_lo_turb), share_up_turb , share_lo_turb) # Seen, die keinen Untersee haben, bekommen fuer den Untersee denselben Fuellstand wie im Obersee

lo_turb             <- sum_turb_lo[idx_turb_lo,2]
lo_pump             <- sum_pump_lo[idx_pump_lo, 2]
lo_spil <- sum_spil_turb_lo[idx_turb_lo,2]

share_pump_lo       <- share_lo[idx_pump_lo]

# Wo kein unterer See vorhanden ist, wird einfach noch einmal die installierte Leistung genommen, so wirkt es nicht als Begrenzung
idx_lo_NA           <- which(is.na(lo_turb))
lo_turb[idx_lo_NA]  <- turb_power[idx_lo_NA,i]     
idx_lo_pump_NA      <- which(is.na(lo_pump))
lo_pump[idx_lo_pump_NA]   <- pump$pump_pinst[idx_lo_pump_NA]

lo_spil <- ifelse(is.na(lo_spil), 0, lo_spil)

# Zusaetzlich zum Reservoirinhalt gibt es noch unregulierten Zufluss in das Wasserkraftwerk, der genutzt werden kann. 
# Dieser Zufluss wird nicht gespeichert. Er geht verloren, wenn er nicht genutzt wird.
# Der unregulierte Zufluss muss noch in Energieeinheiten umgerechnet werden
u_flo_turb          <- u_flo * flo[i]
u_flo_turb_en       <- u_flo_turb* efactor_upper[idx_up_turb_rev] * 1000 / energy_factor

# -------------------------------------
# Das maximale Angebot wird bestimmt.
# Das Angebot wird nicht nur durch die installierte Leistung begrenzt sondern 
# auch durch das verfuegbare Wasser zum produzieren (oben) oder 
# pumpen (unten). Zusaetzlich soll auch noch genug Platz im jeweils anderen See
# (unten fuer Turbinen, oben fuer Pumpen) sein, damit kein Wasser 
# verloren geht. Das gilt nicht fuer die Speicherkraftwerke, da ist das Wasser 
# ohnehin weg.

# Einsatzreihenfolge der Kraftwerke

share_up_turb            <- ifelse(share_up_turb > 1, 1, share_up_turb) # Fuellstaende > 1 koennen nicht zugeordnet werden.
share_lo_turb            <- ifelse(share_lo_turb > 1, 1, share_lo_turb)

share_pump          <- share_pump_up * (1-share_pump_lo)

# Preise
# Fuer die deutschen Speicherkraftwerke werden die Preise anhand einer 
# mathematischen Kurve aus den relativen Fuellstaenden berechnet.

Cmar_turb <- -1756.4*(share_up_turb)^5 + 4946.5*(share_up_turb)^4 - 5499.6*(share_up_turb)^3 +
  2965.2*(share_up_turb)^2 - 780.58*(share_up_turb) + 133.43

# Die Preise fuer Norwegen und Schweden werden berechnet.

# Kombinierter spillage Indikator fuer Ueber- und Untersee pro Kraftwerk
spil_both <- up_spil[idx_turb_scand] - 1.2 * lo_spil[idx_turb_scand]

# Der Angebotspreis wird nach festgelegten Koordinaten von Ueberlauf und 
# Preis approximiert

Cmar_turb_scand <- approx(x_spil, y_price, method = 'linear', rule = 2, xout = spil_both)$y

Cmar_turb[idx_turb_scand] <- Cmar_turb_scand

cmar_turb_matrix[i,] <- Cmar_turb
spil_both_matrix[i,] <- spil_both

# Der Teillastbetrieb der Kraftwerke wird aus Fuellstand und Zufluss des 
# Obersees bestimmt.
part_load_share <- (share_up_turb + share_flo_up_turb)

part_load <- ifelse(part_load_share < 0.2, 0.2, part_load_share)
part_load <- ifelse(part_load_share > 0.7, 0.7, part_load)

# Angebotene Menge
bid_turb            <- pmin (turb_power[,i]*part_load, up_turb, (lo_turb - u_flo_turb_en)) # maximales Angebot der Speicherkraftwerke

# Wenn der Obersee eines KRaftwerks voll ist, wird keine Teillast angenommen
idx_upper_full      <- which(share_up_turb >= 1)
bid_turb[idx_upper_full]  <- pmin (turb_power[idx_upper_full, i], up_turb[idx_upper_full])
  
bid_pump            <- pmin (pump$pump_pinst, up_pump, lo_pump)     # maximales Angebot der Pumpen 

# Zur Sicherheit werden die Gebote noch nach unten bei Null begrenzt.
idx_check1              <- which(bid_pump < 0)
bid_pump[idx_check1] <- 0
idx_check2              <- which(bid_turb < 0)
bid_turb[idx_check2] <- 0

# data.frame erstellen
turb_data           	<- data.frame (turb$turb_nr, bid_turb, Cmar_turb, turb$turb_reg)
colnames(turb_data) 	<- c("unit","Pinst","Cmar","reg_pp") 

pump_data             <- data.frame(pump$pump_nr, bid_pump, share_pump, pump$pump_reg)
colnames(pump_data)   <- c("pump", "bid_pump","share_pump", "reg_pump")                                         

