# Name: code_R_prep_storage
# zuletzt geaendert von gesine
# am 150513

# Hier werden die Speicherberechnungen vorbereitet, die Namen und installierten Leistungen definiert.
# Input: Namen, installierte Leistungen und weitere Angaben zu Wasserkraftwerken aus der Datenbank

# -----   Output
# Turbinen
# turb            - Nummern aller Turbinen, nach stor_nr sortiert
# turb_pinst      - installierte Leistung der Turbinen [MW] nach stor_nr sortiert
# turb_reg        - Region/Laender der Turbinen

# Pumpen
# pump            - Nummern aller Pumpen, nach stor_nr sortiert
# pump_pinst      - installierte Leistung aller Pumpen
# pump_reg        - Regionen/Laender der Pumpen

# Reservoire
# res             - Vektor mit allen Reservoir-Nummern
# fil_max         - maximaler Reservoir-Fuellstand in mio m?
# spil_to         - gibt fuer jedes Reservoir das neachst-niedrigere an
# fil             - Matrix mit Zeitschritten in Zeilen und Reservoiren in Spalten, da werden die Fuellstande reingeschrieben
#                   Der Anfangsfuellstand fuer den Zeitschritt 1 ist mit 0.7 des maximalen Fuellstands festgelegt. Wenn man sp?ter im Jahr anfaengt, wird der entsprechende Fuellstand aus der Norm-Fuellstandskurve genommen.

# Storage - Reservoir - Verbindungen
# turb_upper      - Nummern aller Turbinen die in der Verknuepfungstabelle upper vorkommen, koennen auch mehrmals vorkommen   
# res_upper       - Nummern aller Reservoire die in der Verknuepfungstabelle upper vorkommen, koennen auch mehrmals vorkommen   
# efactor_upper   - electricity factor zu jeder Verbindung von Turbine und Reservoir
# turb_per_res_upper - wenn eine res_nr mehrmals vorkommt, Anteil der Turbinen am Reservoir
# pump_per_res_upper - Analog Anteil der Pumpen am Reservoir (nur Pumpen gerechnet)
# res_per_turb_upper - Anteil eines Reservoirs an der Produktion einer Turbine
# turb_lower      - Nummern aller Turbinen die in der Verknuepfungstabelle lower vorkommen, koennen auch mehrmals vorkommen   
# res_lower       - Nummern aller Reservoire die in der Verknuepfungstabelle lower vorkommen, koennen auch mehrmals vorkommen   
# efactor_lower   - electricity factor zu jeder Verbindung von Turbine und Reservoir
# turb_per_res_lower - wenn eine res_nr mehrmals vorkommt, Anteil der Turbinen am Reservoir
# pump_per_res_lower - Analog Anteil der Pumpen am Reservoir (nur Pumpen gerechnet)
# res_per_turb_lower - Anteil eines Reservoirs an der Produktion einer Turbine

# Zufluss-Daten
# res_flo         - jaehrliche Zuflusssumme fuer jedes Reservoir
# flo             - normalisierte Zuflusskurve, Viertelstundenwerte

###### Daten einlesen
# -----------------------
# storage_scenario einlesen
sql_st_scenario   <- paste("SELECT country_id, capacity FROM storage_scenario WHERE scenario_name = '",storage_scenario,"' AND technology = 'pump' ORDER by country_id", sep = "")

st_scenario 		<- dbGetQuery (con_scenarios, sql_st_scenario)

idx 			      <- match(country_list, st_scenario$country_id) # Die richtige area wird ausgewaehlt.

stor_pot 		    <- as.numeric(st_scenario$capacity[idx])
stor_pot[which(is.na(stor_pot) == TRUE)] 		<- 0

# -----------------------
quo_scenario 		<- dbGetQuery(con_scenarios, "SELECT country_id, capacity FROM storage_scenario WHERE scenario_name = 'status_quo' AND technology = 'pump' ORDER by country_id")

idx 			      <- match(country_list, quo_scenario$country_id)
stor_quo 		    <- as.numeric(quo_scenario$capacity[idx])
stor_quo[which(is.na(stor_quo) == TRUE)] 		<- 0

# Vektor fuer nur den Ausbau
stor_ext 		<- (stor_pot - stor_quo)/1000

# Andere Speicher ausser Wasserspeichern
sql_st_other_scenario   <- paste("SELECT country_id, capacity, energy, efficiency FROM storage_scenario WHERE scenario_name = '",storage_scenario,"' AND technology = 'other' ORDER by country_id", sep = "")
st_other_scenario   	<- dbGetQuery (con_scenarios, sql_st_other_scenario)

idx_stor 			  <- match(country_list, st_other_scenario$country_id)
stor_pot_oth 		<- as.numeric(st_other_scenario$capacity[idx_stor])
en_pot_oth      <- as.numeric(st_other_scenario$energy[idx_stor])
other_eff       <- as.numeric(st_other_scenario$efficiency[idx_stor])

stor_pot_oth [which(is.na(stor_pot_oth) == TRUE)] 		<- 0
en_pot_oth   [which(is.na(en_pot_oth) == TRUE)]       <- 0
other_eff    [which(is.na(other_eff) == TRUE)]        <- 0

# dem gewaehlten scenanario_year anpassen
stor_oth_factor 	<- (scenario_year - 2011) / (2050 - 2011)
stor_pot_oth 		  <- stor_pot_oth * stor_oth_factor
en_pot_oth 		    <- en_pot_oth * stor_oth_factor

stor_oth_nr     <- country_list*10000 + 4001

# auf Regionen in Deutschland aufteilen
other_share   <- c(0.2, 0.2, 0.2, 0.2, 0.2)
stor_pot_other_de <- stor_pot_oth[1] * other_share
en_pot_oth_de <- en_pot_oth[1] * other_share

stor_nr_de <- c(14001, 14002, 14003, 14004, 14005)

stor_pot_oth <- c(stor_pot_other_de, stor_pot_oth[2:9])
en_pot_oth <- c(en_pot_oth_de, en_pot_oth[2:9])
stor_oth_nr <- c(stor_nr_de, stor_oth_nr[2:9])
other_eff <- c(rep(other_eff[1], 4), other_eff)

other_regions    <- country_list*100
other_regions    <- c(102, 101, 106, 109, 117, other_regions[2:9])

# Grenzkosten der Turbinen aus der Datenbank auslesen, die Grenzkosten sind jeweils einem relativen Reservoirfuellstand zugeordnet von 1.00 bis 0.00, fuer Deutschland 

Cmar_turb 	<- dbGetQuery (con_baltic, "SELECT rel_fil, Cmar from storage_cmar")
rel_fil 	  <- as.numeric(as.character(Cmar_turb$rel_fil))
Cmar_turb 	<- as.numeric(as.character(Cmar_turb$Cmar))

#-------------------------
# Neue Speicherleistung
new_pump        <- dbGetQuery(con_baltic, "SELECT stor_nr, pinst, dpr, year FROM storage WHERE type = 'NP' ORDER BY stor_nr")
new_pump_nr             <- c(as.numeric(as.character(new_pump$stor_nr)))
new_pump_pinst          <- c(as.numeric(as.character(new_pump$pinst)))
new_pump_reg            <- c(as.numeric(as.character(new_pump$dpr)))
new_pump_year           <- c(as.numeric(as.character(new_pump$year)))

new_pump                <- data.frame(new_pump_nr, new_pump_pinst, new_pump_reg, new_pump_year)
  
pump_ext                <- vector("list", length(country_list))
new_turb        <- dbGetQuery(con_baltic, "SELECT stor_nr, pinst, dpr, year FROM storage WHERE type = 'NPK' ORDER BY stor_nr")
new_turb_nr             <- c(as.numeric(as.character(new_turb$stor_nr)))
new_turb_pinst          <- c(as.numeric(as.character(new_turb$pinst)))
new_turb_reg            <- c(as.numeric(as.character(new_turb$dpr)))
new_turb_year           <- c(as.numeric(as.character(new_turb$year)))
  
new_turb                <- data.frame(new_turb_nr, new_turb_pinst, new_turb_reg, new_turb_year)
  
turb_ext                <- vector("list", length(country_list))

last_new_turb_nr        <- numeric(length(country_list))
  
# Die Speicher ausweahlen, die den Szenario-Parameter stor_ext erfuellen. Das passiert innerhalb einer Regionenschleife
  
for (r in country_list){
  idx_country             <- which(r == country_list)
  if (stor_ext[idx_country]> 0){
    pump_ext_reg          <- subset(new_pump, new_pump$new_pump_reg %in% country_reg[[r]])
    turb_ext_reg          <- subset(new_turb, new_turb$new_turb_reg %in% country_reg[[r]])
    
    cum_new_pump          <- cumsum(pump_ext_reg$new_pump_pinst)
    mar 			            <- f_marginal_idx(stor_ext[idx_country]*1000,cum_new_pump,pump_ext_reg$new_pump_nr)
    idx_pump_new          <- as.numeric(as.character(mar[,1]))

    pump_ext_reg          <- pump_ext_reg[1:idx_pump_new,]
    turb_ext_reg          <- turb_ext_reg[1:idx_pump_new,]
    
    if(idx_pump_new > 1) {
      pump_ext_reg$new_pump_pinst[idx_pump_new]  <- stor_ext[idx_country]*1000 - cum_new_pump[idx_pump_new - 1]
      turb_ext_reg$new_turb_pinst[idx_pump_new]  <- stor_ext[idx_country]*1000 - cum_new_pump[idx_pump_new - 1]
      } else {
      pump_ext_reg$new_pump_pinst[idx_pump_new]  <- stor_ext[idx_country]*1000 
      turb_ext_reg$new_turb_pinst[idx_pump_new]  <- stor_ext[idx_country]*1000
    }
    
    pump_ext[[idx_country]]         <- pump_ext_reg
    turb_ext[[idx_country]]         <- turb_ext_reg    
      
  }else{
  
      }
}

new_turb                  <- do.call("rbind", turb_ext)
new_pump                  <- do.call("rbind", pump_ext)

#-------------------------
# Turbinen
turb          <- dbGetQuery (con_baltic, "SELECT stor_nr, pinst, dpr , year FROM storage WHERE `type` = 'PK' or type = 'K'  ORDER BY `stor_nr`")
turb_nr                 <- c(as.numeric(as.character(turb$stor_nr)))
turb_pinst              <- c(as.numeric(as.character(turb$pinst))) 
turb_reg                <- c(as.numeric(as.character(turb$dpr)))
turb_year               <- c(as.numeric(as.character(turb$year)))

turb              <- data.frame(turb_nr, turb_pinst, turb_reg, turb_year)
turb              <- subset(turb, turb_reg %in% reg_vector) # Richtige Region raussuchen

# ----------------------
# Pumpen
pump              <- dbGetQuery (con_baltic, "SELECT stor_nr, pinst, dpr, year FROM storage WHERE `type` = 'P' ORDER BY stor_nr")
pump_nr                 <- c(as.numeric(as.character(pump$stor_nr)))
pump_pinst              <- c(as.numeric(as.character(pump$pinst)))
pump_reg                <- c(as.numeric(as.character(pump$dpr)))
pump_year               <- c(as.numeric(as.character(pump$year)))
  
pump              <- data.frame(pump_nr, pump_pinst, pump_reg, pump_year)
pump              <- subset(pump, pump$pump_reg %in% reg_vector)

# Die neuen Turbinen und Pumpen werden an die alten angehaengt.
if (sum(stor_ext>0)){
  colnames(new_turb)  <- c("turb_nr", "turb_pinst", "turb_reg", "turb_year")
  turb                <- rbind(turb, new_turb)
  
  colnames(new_pump)  <- c("pump_nr", "pump_pinst", "pump_reg", "pump_year")
  pump                <- rbind(pump, new_pump)
  
}

for (r in country_list){
  idx_country             <- which(r == country_list)
  turb_reg <- subset(new_turb, new_turb$turb_reg %in% country_reg[[r]] )
  if(length(turb_reg$turb_nr) == 0){
    last_new_turb_nr[idx_country] <- 0
  }else{
    last_new_turb_nr[idx_country]<- max(turb_reg$turb_nr)
  }
}

# ----------------------
# Reservoire
res                 <- dbGetQuery (con_baltic, "SELECT res_nr, fil_max , total_efactor FROM reservoirs ORDER BY res_nr")

# Die Reservoire der gerechneten area raussuchen
res_list            <- vector("list", length(country_list))
for (r in country_list) {
  idx_country           <- which(r == country_list)
  res_part              <- subset(res, res$res_nr > r*1000 & res$res_nr < (r+1)*1000)
  res_list[[idx_country]] <- res_part
}

res                     <- do.call("rbind", res_list)
  
res_nr                  <- c(as.numeric(as.character(res$res_nr)))
fil_max                 <- c(as.numeric(as.character(res$fil_max)))
total_efactor           <- c(as.numeric(as.character(res$total_efactor)))

idx_res_de              <- which(res_nr < 2000)

# Der Fluss in Fluessen in das naechste Reservoir, in der Reihenfolge von storage 
stor_river_res      <- dbGetQuery(con_baltic, "SELECT stor_nr, flo_river FROM storage WHERE type != 'P' AND type != 'NP' ORDER BY stor_nr")
stor_river_res      <- subset(stor_river_res, stor_river_res$stor_nr %in% turb$turb_nr)
flo_river_res       <- as.numeric(as.character(stor_river_res$flo_river))
idx_flo_river_NA    <- which(is.na(flo_river_res) == TRUE)
flo_river_res[idx_flo_river_NA] <- 0

idx_flo_river       <- match(res_nr, flo_river_res)

# Die Matrix der Reservoir-Fuellstaende wird initialisiert und fuer den ersten Zeitschritt gefuellt.
fil                 <- matrix(0, nrow = end_timestep + 1, ncol = length(res_nr)) 
fil_other           <- matrix(0, nrow = end_timestep +1, ncol = length(stor_pot_oth))

# --------------------------------
# Die Verbindungen von Reservoiren und Kraftwerken werden eingelesen
res_turb_up         <- dbGetQuery (con_baltic, "SELECT * FROM reservoir_turbine_upper")
stor_upper          <- as.numeric(as.character(res_turb_up$stor_nr))
res_upper           <- as.numeric(as.character(res_turb_up$res_nr))
efactor_upper       <- as.numeric(as.character(res_turb_up$efactor))

res_turb_lo         <- dbGetQuery (con_baltic, "SELECT * FROM reservoir_turbine_lower")
stor_lower          <- as.numeric(as.character(res_turb_lo$stor_nr))
res_lower           <- as.numeric(as.character(res_turb_lo$res_nr))
efactor_lower       <- as.numeric(as.character(res_turb_lo$efactor))

# Die nicht gebrauchten neuen Speicher rauswerfen
upper         <- data.frame(stor_upper, res_upper, efactor_upper)
upper_red     <- subset(upper, upper$stor_upper %in% c(turb$turb_nr, pump$pump_nr))
stor_upper    <- upper_red$stor_upper
res_upper     <- upper_red$res_upper
efactor_upper <- upper_red$efactor_upper

lower         <- data.frame(stor_lower, res_lower, efactor_lower)
lower_red     <- subset(lower, lower$stor_lower %in% c(turb$turb_nr, pump$pump_nr))
stor_lower    <- lower_red$stor_lower
res_lower     <- lower_red$res_lower
efactor_lower <- lower_red$efactor_lower

# Unique Eintraege rausfinden
stor_upper_un       <- unique(stor_upper)
stor_upper_un       <- sort(stor_upper_un, decreasing = FALSE)
res_upper_un        <- unique(res_upper)
res_upper_un        <- sort(res_upper_un, decreasing = FALSE)
  
stor_lower_un       <- unique(stor_lower)
stor_lower_un       <- sort(stor_lower_un, decreasing = FALSE)
res_lower_un        <- unique(res_lower)
res_lower_un        <- sort(res_lower_un, decreasing = FALSE)

# Die Nummern der Reservoire aus den Verbindungstabellen werden gematcht.
idx_upper_res       <- match(res_upper, res_nr)  # Index in der Laenge turb_res_upper
fil_max_up          <- fil_max[idx_upper_res]

idx_lower_res       <- match(res_lower, res_nr)  # Index in der Laenge turb_res_lower
fil_max_lo          <- fil_max[idx_lower_res]

idx_res_upper       <- match(res_nr, res_upper) # Index in der Laenge von re_nr
idx_res_upper_un    <- match(res_nr, res_upper_un)
idx_res_lower       <- match(res_nr, res_lower)
idx_res_lower_un    <- match(res_nr, res_lower_un)

idx_turb_up         <- match(turb$turb_nr, stor_upper_un)
idx_pump_up         <- match(pump$pump_nr, stor_upper_un)
                                                              
idx_turb_lo         <- match(turb$turb_nr, stor_lower_un)
idx_pump_lo         <- match(pump$pump_nr, stor_lower_un)

# Der Anteil der Reservoire pro Kraftwerk wird berechnet.
idx_sum_up          <- match(stor_upper, stor_upper_un)
idx_sum_lo          <- match(stor_lower, stor_lower_un)

# Der Anteil der Kraftwerke pro Reservoir
# Oberseen
idx_up_turb         <- match(stor_upper, turb$turb_nr)
idx_up_turb_rev     <- match(turb$turb_nr, stor_upper)

pinst_turb_up       <- turb$turb_pinst[idx_up_turb]
idx_turb_up_NA      <- which(is.na(pinst_turb_up)== TRUE)
pinst_turb_up[idx_turb_up_NA] <- 0

idx_up_pump         <- match(stor_upper, pump$pump_nr)
pinst_pump_up       <- pump$pump_pinst[idx_up_pump]
idx_pump_up_NA      <- which(is.na(pinst_pump_up) == TRUE)
pinst_pump_up[idx_pump_up_NA] <- 0

sum_pinst_turb_up   <- aggregate(pinst_turb_up, list(res_upper), sum) 
# Installierte Leistung wird pro Reservoir aufsummiert.

idx_pinst_up        <- match(res_upper, res_upper_un)
turb_per_res_upper  <- pinst_turb_up/sum_pinst_turb_up[idx_pinst_up,2]
idx_turb_res_up     <- which(is.na(turb_per_res_upper) == TRUE)
turb_per_res_upper[idx_turb_res_up] <- 0

sum_pinst_pump_up   <- aggregate(pinst_pump_up, list(res_upper), sum)
pump_per_res_upper  <- pinst_pump_up/sum_pinst_pump_up[idx_pinst_up,2]
idx_pump_res_up     <- which(is.na(pump_per_res_upper) == TRUE)
pump_per_res_upper[idx_pump_res_up] <- 0

sum_fil_max_up      <- aggregate(fil_max_up, list(stor_upper), sum) 
# Die Speicherkapazitaet wird pro Speicher aufsummiert, fuer oben

sum_fil_max_lo      <- aggregate(fil_max_lo, list(stor_lower), sum) 
# Die Speicherkapazitaet wird pro Speicher aufsummiert, fuer unten

# Unterseen
idx_lo_turb         <- match(stor_lower, turb$turb_nr)
pinst_turb_lo       <- turb$turb_pinst[idx_lo_turb]
idx_turb_lo_NA      <- which(is.na(pinst_turb_lo) == TRUE)
pinst_turb_lo[idx_turb_lo_NA] <- 0

idx_lo_pump         <- match(stor_lower, pump$pump_nr)
pinst_pump_lo       <- pump$pump_pinst[idx_lo_pump]
idx_pump_lo_NA      <- which(is.na(pinst_pump_lo) == TRUE)
pinst_pump_lo[idx_pump_lo_NA] <- 0

sum_pinst_turb_lo   <- aggregate(pinst_turb_lo, list(res_lower), sum)
idx_pinst_lo        <- match(res_lower, res_lower_un)
turb_per_res_lower  <- pinst_turb_lo/sum_pinst_turb_lo[idx_pinst_lo,2]
idx_turb_res_lo     <- which(is.na(turb_per_res_lower) == TRUE)
turb_per_res_lower[idx_turb_res_lo] <- 0

sum_pinst_pump_lo   <- aggregate(pinst_pump_lo, list(res_lower), sum)
pump_per_res_lower  <- pinst_pump_lo/sum_pinst_pump_lo[idx_pinst_lo,2]
idx_pump_res_lo     <- which(is.na(pump_per_res_lower) == TRUE)
pump_per_res_lower[idx_pump_res_lo] <- 0

# Index fuer bestimmte Laender
idx_turb_no         <- which(turb$turb_nr > 90000) # Nur die norwegischen Speicher haben einen unregulierten Zufluss direkt in die  Kraftwerke
idx_turb_scand      <- which(turb$turb_nr > 90000 | (turb$turb_nr > 70000 & turb$turb_nr < 80000)) # Die Speicher in Norwegen und Schweden haben eine andere Preisbildung.

# -------------------------------------
# Zufluss Daten
# Es wird unterschieden zwischen dem regulierten Zufluss, der in die Reservoire fliesst und dem unregulierten, der direkt zu den Kraftwerken fliesst.
# Regulierter Zufluss
res_flo             <- dbGetQuery (con_baltic, "SELECT res_nr, flo FROM reservoirs ORDER BY res_nr")
res_flo_part        <- subset(res_flo, res_flo$res_nr %in% res$res_nr)
res_flo             <- as.numeric(as.character(res_flo_part$flo))

if (area != "de"){
  # Unregulierter Zufluss (nur in Norwegen)
  idx_country_no    <- which(country_list == 9)
  sql_u_flo         <- paste("SELECT SUM(flo) 
                              FROM storage 
                              LEFT OUTER JOIN storage_inflow_field_no 
                              ON storage.stor_nr = storage_inflow_field_no.hpp_nr
                              AND storage_inflow_field_no.res_nr = '0' 
                              WHERE (type LIKE 'K' OR type LIKE 'PK' 
                                    OR (type LIKE 'NPK' AND storage.stor_nr <= '",last_new_turb_nr[idx_country_no],"')) 
                              AND stor_nr > 90000 
                              GROUP BY storage.stor_nr" , sep="")
  
  u_flo_no            <- dbGetQuery(con_baltic, sql_u_flo)
  u_flo_no            <- as.numeric(as.character(unlist(u_flo_no)))
  idx_NA_flo          <- which(is.na(u_flo_no)== TRUE)
  u_flo_no[idx_NA_flo]<- 0
  
  u_flo               <- rep.int(0, times = length(turb$turb_nr))
  
  u_flo[idx_turb_no]  <- u_flo_no

} else {
  
  u_flo               <- rep.int(0, times = length(turb$turb_nr))
}

# Hier bestimmt die Auswahl des Wetterjahres das Jahr der Zuflussdaten

# if (weather_year == 2010){flo_year <- 1969 } # schlechtes Wasserjahr 72%
# if (weather_year == 1998){
# flo_year <- 1990  # gutes Wasserjahr 137%
# if (weather_year == 2003){
flo_year <- 1979  # mittleres Wasserjahr
        
# Fuer den Zufluss der Reservoire wird im Moment angenommen, dass die Zuflusskurve fuer alle Reservoire gleich ist.
# Sie ist in der Tabelle flo hinterlegt und wird fuer jedes Reservoir skaliert.                    
sql_flo             <- paste("SELECT flo FROM storage_flow_",time_unit," WHERE year = '",flo_year,"' ORDER BY ",time_unit,"", sep="")
flo                 <- dbGetQuery (con_baltic, sql_flo)
flo                 <- as.numeric(as.character(unlist(flo)))

# Unregulierter Must-Run
u_energy <- u_flo * efactor_upper[idx_up_turb_rev] * 1000 / energy_factor # MW

u_power_quarter <- apply((t(apply(t(u_energy), 2 , function(x) flo * x))), 2, function(x) pmin(turb$turb_pinst, x))

u_flo_reg     <- matrix(0, nrow = timesteps, ncol = region_split)
u_flo_reg[,6] <- colSums(u_power_quarter)

turb_power <- matrix(rep(turb$turb_pinst, each = timesteps), ncol = timesteps, byrow = TRUE)

turb_power <- turb_power - u_power_quarter

rm(u_power_quarter)

# Zuflusskurve in Deutschland
sql_inflow_de       <- paste("SELECT flo FROM storage_flow_",time_unit," WHERE level_meter = 30 ORDER BY ",time_unit,"", sep="") # Station Rheinfelden

inflow_de           <- dbGetQuery(con_baltic, sql_inflow_de)
inflow_de           <- as.numeric(as.character(unlist(inflow_de)))
flo_de              <- inflow_de*sum(flo)/sum(inflow_de) # Kurve wird normiert und dem Wasserjahr angepasst

# Zufluss nach Kraftwerken - fuer die Einsatzreihenfolge
flo_turb                <- res_flo[idx_upper_res]*turb_per_res_upper
sum_flo_turb            <- aggregate(flo_turb, list(stor_upper), sum)
flo_all                 <- sum_flo_turb[idx_turb_up,2] + u_flo # Nur fuer Turbinen
 
share_flo               <- (max(flo)*flo_all) / (turb$turb_pinst*energy_factor*0.001/efactor_upper[idx_up_turb_rev])
share_flo_vol           <- flo_all / sum_fil_max_up[idx_turb_up,2]
share_flo_vol           <- ifelse(share_flo_vol == 0, 0.01, share_flo_vol)

# Fuer jedes Kraftwerk und Reservoir werden alle Ebenen darueber bestimmt
source('hydro_recursion.R')
source('hydro_recursion_res.R')

share_flo_sum       <- max(flo)*(flo_sum + u_flo)/(turb$turb_pinst*energy_factor*0.001/efactor_upper[idx_up_turb_rev])

flo_reg             <- numeric(length(res_nr))

# Natuerlicher Fuellstand in einer Viertelstunde, zur Preisbildung
sql_storage_norm        <- paste("SELECT fil_norm, price_norm FROM storage_fil_norm_",time_unit," ORDER BY ",time_unit,"", sep="")
storage_norm            <- dbGetQuery(con_baltic, sql_storage_norm)                                      
price_norm              <- as.numeric(as.character(storage_norm$price_norm))
fil_norm                <- as.numeric(as.character(storage_norm$fil_norm))              
fil_norm_up_lo          <- fil_norm * (1-fil_norm)                                                  

# Der Anfangswert der Reservoire wird festgelegt.
fil[start_timestep,]     <- fil_max*fil_norm[start_timestep]
fil_other[start_timestep,] <- en_pot_oth * 0.5

elp_stor_other_turb <- matrix(0, nrow = end_timestep, ncol = length(stor_pot_oth))
elp_stor_other_pump <- matrix(0, nrow = end_timestep, ncol = length(stor_pot_oth))

# Fuer den Ueberlauf von Wasser (spil) wird hier festgehalten von welchem zu welchem Reservoir das Wasser fliesst.

spil_to             <- dbGetQuery(con_baltic, "SELECT DISTINCT reservoir_turbine_upper.res_nr, reservoir_turbine_lower.res_nr AS res_lo FROM `reservoir_turbine_upper` LEFT OUTER JOIN reservoir_turbine_lower ON reservoir_turbine_upper.stor_nr = reservoir_turbine_lower.stor_nr GROUP BY reservoir_turbine_upper.res_nr")# Hier ist in der Reihenfolge der Reservoire der naechste Untersee festgehalten.

spil_to             <- subset(spil_to, spil_to$res_nr %in% res_upper) # Die nicht genutzten neuen Speicher kommen raus.
spil_to             <- as.numeric(as.character(spil_to$res_lo))

spil_to_res_nr      <- spil_to[idx_res_upper_un]

idx_NA0             <- which(is.na(spil_to_res_nr) == TRUE) 
spil_to_res_nr[idx_NA0]    <- 0

idx_spil            <- match(res_nr, spil_to_res_nr)
idx_spil            <- ifelse(is.na(idx_spil), idx_flo_river, idx_spil)

spil <- numeric(end_timestep)

cmar_turb_matrix <- matrix(nrow = timesteps, ncol = length(turb$turb_nr))
spil_both_matrix <- matrix(nrow = timesteps, ncol = length(idx_turb_scand))

# Wertepaare von Ueberlauf und Preisen fuer Approximation

x_spil  <- c(-832000, -831922.9, -218565, -92310, -17260,
            -2883, 848.394, 1564.994, 3412.416, 3413)

y_price <- c(60, 47.048, 38.074, 35.6, 31, 25.49, 21.63, 17.872, 10.26, 5)
