# code_R_solar
# zuletzt bearbeitet von gesine
# 061012

# In diesem Code-Stueck wird die Solar-Winspeisekurve berechnet aus den Einstrahlungsdaten. Es wird eine (timesteps x region_split)-Matrix "solar_reg "in MW erstellt

#----------------------------
solar_env <- function(){
# weather_data einlesen, die Laendernummer wird aus der weather_station Tabelle zusaetzlich zugeordnet
sql_solar   	<- paste("SELECT weather_data.station_id, weather_station.country_id, hour, solar_radiation FROM weather_data, weather_station WHERE year = ",weather_year,"AND hour >=", start_timestep_weather,"AND hour <=",end_timestep_weather,"AND weather_data.station_id = weather_station.station_id", sep = " ")
solar_data 		<- dbGetQuery(con_baltic, sql_solar)

# Stationsdaten einlesen zur Bestimmung der Gewichtung und ob Stark- oder Schwachwind
sql_station   	<- paste("SELECT station_id, country_id, weight_solar FROM weather_station", sep = " ")
station_data 		<- dbGetQuery(con_baltic, sql_station)

#----------------------------

# in Liste bringen fuer die Berechnung der Einspeise-Zeitreihen, eine Matrix pro Land, Stunden in den Reihen, Stationen in den Spalten

solar_list 	<- vector("list", 1 + region_abroad)

solar_reg <- matrix(0, nrow = end_timestep_weather, ncol = 1 + region_abroad)

for(country in country_list){
  idx_country       <- which(country_list == country)
  weather_part 			<- subset(solar_data, country_id == country)
  weather_part 			<- weather_part[order(weather_part$hour),]
  weather_part 			<- weather_part[order(weather_part$station_id),]
  stations 			<- unique(weather_part$station_id)
  weather_matrix 		<- matrix(weather_part$solar_radiation, ncol = length(stations), nrow = timesteps_weather)
  colnames(weather_matrix) 	<- stations
  solar_list[[idx_country]] 		<- weather_matrix

  if (country != 9) {
    
  # Hoirzontale Globalstrahlung zur Einspeiseleistung pro Station berechnen
  station_part  <- subset(station_data, country_id == country)
  station_part  <- station_part[order(station_part$station_id),]
  P_inst        <- as.numeric(as.character(scenario_re$solar[which(scenario_re$country_id==country)])) * 1000 # aus GW werden MW
  
  solar_matrix <- matrix(NA,ncol=length(stations), nrow = timesteps_weather)
  
  for(station in 1:length(stations)){
    # Einspeiseleistung [MW] = globale Einstrahlung [kWh/m?] * Gewichtung * installierte Leistung * Verfuegbarkeit [MW (bei 1000 W/m? = 1 kWh/m?)]
    solar_matrix[,station] <- weather_matrix[,station] * station_part$weight_solar[station] * P_inst * .98
    solar_matrix[which(solar_matrix[,station]>P_inst),station]<-P_inst
  }
  solar_reg[start_timestep_weather:end_timestep_weather, idx_country] <- rowSums(solar_matrix)
  
  } else {
   solar_reg[,idx_country]  <- 0
  }
  
}

# Wenn die Rechnung fuer Viertelstunden gemacht wird, muessen die Daten interpoliert werden.
if (time_unit == "quarter") {
  solar_reg_quarter                     <- matrix(0, ncol = length(country_list), nrow = end_timestep)
  idx_hour                              <- seq.int(1,(end_timestep - 3), by = 4)
  diff_hour                             <- diff(solar_reg)
  diff_hour                             <- rbind(diff_hour, diff_hour[timesteps_weather-1,]) # Die letzte Differenz wird zweimal verwendet
  
  solar_reg_quarter[idx_hour,] <- solar_reg
  
  solar_reg_quarter[idx_hour + 1, ]  <- solar_reg_quarter[idx_hour, ] + diff_hour/4
  solar_reg_quarter[idx_hour + 2, ]  <- solar_reg_quarter[idx_hour, ] + diff_hour/4*2
  solar_reg_quarter[idx_hour + 3, ]  <- solar_reg_quarter[idx_hour, ] + diff_hour/4*3
  
  solar_reg <- solar_reg_quarter
}
return(solar_reg)
}
solar_reg <- solar_env()

# -----------------------------------------------------------------------------
  # Solardaten in Deutschland
solar_de_env <- function(){
source('code_R_solar_de.R', local = TRUE) # Aus diesem Code-Stueck kommt solar_dpr_de, eine Matrix (timesteps x dpr), Stunden oder Quarter je nach Berechnung
  
  solar_reg_de <- matrix(NA, ncol = region_split_de, nrow = end_timestep)
  
  if (region_split_de < 21) {
    # die Aufteilung der dpr auf die Regionen ist in region_list gespeichert
    for(reg in 1:region_split_de){
      r <- region_list[[reg]]
      if(length(r) == 1){
        solar_reg_de[,reg] <- solar_dpr_de[,(region_list[[reg]]-100)]
      }else{
        solar_reg_de[,reg] <- rowSums(solar_dpr_de[,(region_list[[reg]]-100)])
      }
    }
  } else {
    solar_reg_de <- solar_dpr_de
  }

  if(length(country_list)==1){
    solar_reg <- solar_reg_de
  }else{
    solar_reg <- cbind(solar_reg_de, solar_reg[ , 2:length(country_list)])
  }

return(solar_reg)
}
solar_reg <- solar_de_env()

