# code_R_wind_off
# zuletzt bearbeitet von frauke
# 201012

# In diesem Code-Stueck wird die Offshore-Windkurve berechnet aus den Windmessungsdaten. Es wird eine (timesteps x region_split)-Matrix "wind_reg_off "in MW erstellt

#----------------------------
# Fuer die Wetterzeitreihen wird im Moment nur mit Stunden gerechnet.
# Wenn die time_unit = quarter ist, werden hier die Zeitschritte * 0.25 gerechnet. So wird die richtige Anzahl an Stunden ausgelesen. Fuer den Fall, dass die Zahl sich nicht durch 4 teilen laesst, wird auf die volle Stunde aufgerundet.
timesteps_weather       <- round(timesteps * energy_factor, digits = 0) 
end_timestep_weather    <- round(end_timestep * energy_factor, digits = 0)
start_timestep_weather  <- round(start_timestep * energy_factor, digits = 0)

#----------------------------
wind_off_env <- function(){
# weather_data einlesen, die Laendernummer wird aus der weather_station Tabelle zusaetzlich zugeordnet
sql_wind   	  <- paste("SELECT weather_data.station_id, weather_station.country_id, hour, wind_speed, wind_offshore FROM weather_data, weather_station WHERE year = ",weather_year,"AND hour >=" , start_timestep_weather,"AND hour <=",end_timestep_weather,"AND weather_data.station_id = weather_station.station_id", sep = " ")
wind_data 		<- dbGetQuery(con_baltic, sql_wind)

# Offshore-Fino-Daten einlesen
sql_fino      <- paste("SELECT fino_id, year, hour, wind_speed FROM weather_offshore WHERE hour <=",end_timestep_weather, sep = " ")
fino_data 		<- dbGetQuery(con_baltic, sql_fino)

#----------------------------

# in Liste bringen fuer die Berechnung der Einspeise-Zeitreihen, ein Vektor pro Land, Stunden in den Reihen

wind_list 	  <- vector("list", 1 + region_abroad)
wind_off_reg  <- matrix(0, nrow = end_timestep_weather, ncol = 1 + region_abroad)
korrektur <- c(1,1,1.1,1,1,.8,.85,.75,1) # fuer verschiedene rauhigkeiten um die messstationen

for(country in country_list){
  idx_country       <- which(country_list == country)
  weather_part 			<- subset(wind_data, country_id == country)
  weather_part   		<- subset(weather_part, wind_offshore == 1)
  weather_part 			<- weather_part[order(weather_part$hour),]
  stations 			    <- unique(weather_part$station_id)
  weather_matrix 		<- matrix(weather_part$wind_speed, ncol = 1, nrow = timesteps_weather)
  colnames(weather_matrix) 	<- stations
  wind_list[[idx_country]]   <- weather_matrix
  
  # Windgeschwindigkeit (in 10m Hoehe) Ueber Leistungskennlinien zur Einspeiseleistung fuer die Offshore-Station berechnen
  P_inst          <- as.numeric(as.character(scenario_re$wind_offshore[which(scenario_re$country_id==country)])) * 1000 # aus GW werden MW
  wind_off_matrix <- matrix(NA, ncol=1, nrow = timesteps_weather)
  
  # Leistungskennlinie Offshore: Starkwind-Leistungskurve (Siemens SWT-3,6) in kW  
  x<-c(2.9,3,4,5,6,7,8,9,10,11,12,13,14,24.99,25)
  y<-c(0,80,238,474,802,1234,1773,2379,2948,3334,3515,3577,3600,3600,0)
  
  # Approximieren ueber Leistungskennlinie / Nennleistung * installierter Leistung
  #Anpassen der Windgeschwindigkeit auf Nabenhoehe. Bei Litauen zusaetzlich 20%
  wind_off_matrix <- approx(x, y, xout=(weather_matrix*1.6*korrektur[country]), method='linear', rule = 2)$y / 3600 * P_inst 
  
  wind_off_reg[start_timestep_weather:end_timestep_weather, idx_country] <- wind_off_matrix
  
}

  #--------------------------
  # Fino Daten nutzen fuer Deutschland (Fino1 und 2), Polen (Fino2), Schweden (Fino2),  Daenemark(Fino2 und 3) und Norwegen (Fino4 = Norwegische Offshore Daten von Sleipner)
  fino_data$wind_power <- approx(x, y, xout=fino_data$wind_speed, method='linear', rule = 2)$y / 3600
  
  Pfino         <- numeric(length(country_list))
  
  for(land in country_list){
    Pfino[land] <- as.numeric(as.character(scenario_re$wind_offshore[which(scenario_re$country_id==land)])) * 1000
  }
  
  fino <- matrix(NA, nrow = end_timestep_weather, ncol=4)
  
  for(fino_station in 1:4){ 
    fino[,fino_station] <- fino_data$wind_power[which(fino_data$fino_id==fino_station)]
  }
  
  wind_off_reg[,1] <- (wind_off_reg[,1] + fino[,1]*Pfino[1] + fino[,2]*Pfino[1]) / 3
  
  if (area == "no_de"){
    wind_off_reg[,2] <- ((wind_off_reg[,2]) + fino[,4])*Pfino[2]/2
  }
  
  if (area == "baltic"){
    wind_off_reg[,2] <- (wind_off_reg[,2] + fino[,2]*Pfino[2]) / 2
    wind_off_reg[,7] <- (wind_off_reg[,7] + fino[,2]*Pfino[7]) / 2
    wind_off_reg[,8] <- (wind_off_reg[,8] + fino[,2]*Pfino[8] + fino[,3]*Pfino[8]) / 3
    wind_off_reg[,2] <- ((wind_off_reg[,2]) + fino[,4])*Pfino[2]/2
  }

#----------------------------
if (region_split_de > 1) {
  # Verteilung der Windeinspeisung auf die Regionen innerhalb Deutschlands
  wind_off_dpr_de             <- matrix(0, nrow = timesteps_weather, ncol = 21) 
  wind_off_share              <- c(0.1, 0.3, 0.6) # Anteile der drei offshore Regionen 19, 20, 21
  wind_off_dpr_de[ ,19:21]    <- wind_off_reg[ ,1] %*% t(wind_off_share) 
  
  wind_off_reg_de <- matrix(NA, ncol = region_split_de, nrow = timesteps_weather)
  
  if (region_split_de < 21) {
    # die Aufteilung der dpr auf die Regionen ist in region_list gespeichert
    for(reg in 1:region_split_de){
      r <- region_list[[reg]]
      if(length(r) == 1){
        wind_off_reg_de[,reg] <- wind_off_dpr_de[,(region_list[[reg]]-100)]
      }else{
        wind_off_reg_de[,reg] <- rowSums(wind_off_dpr_de[,(region_list[[reg]]-100)])
      }
    }
  } else {
    wind_off_reg_de           <- wind_off_dpr_de
  }
  
  if(length(country_list)>1){
    wind_off_reg <- cbind(wind_off_reg_de, wind_off_reg[ , 2:length(country_list)])
  }else{
    wind_off_reg <- wind_off_reg_de
  }
  
} 

#----------------------------
# Wenn die Rechnung fuer Viertelstunden gemacht wird, muessen die Daten interpoliert werden.
if (time_unit == "quarter") {
  wind_off_reg_quarter    <- matrix(0, ncol = region_split, nrow = end_timestep)
  idx_hour                <- seq.int(1,(end_timestep - 3), by = 4)
  diff_hour               <- diff(wind_off_reg)
  diff_hour               <- rbind(diff_hour, diff_hour[timesteps_weather-1,]) 
  # Die letzte Differenz wird zweimal verwendet
  
  wind_off_reg_quarter[idx_hour,] <- wind_off_reg
  
  wind_off_reg_quarter[idx_hour + 1, ]  <- wind_off_reg_quarter[idx_hour, ] + diff_hour/4
  
  wind_off_reg_quarter[idx_hour + 2, ]  <- wind_off_reg_quarter[idx_hour, ] + diff_hour/4*2
  
  wind_off_reg_quarter[idx_hour + 3, ]  <- wind_off_reg_quarter[idx_hour, ] + diff_hour/4*3
  
  wind_off_reg <- wind_off_reg_quarter
}
return(wind_off_reg)
}
wind_off_reg <- wind_off_env()
