# code_R_wind_on
# zuletzt bearbeitet von frauke
# 201012

# In diesem Code-Stueck wird die Onshore-Windkurve berechnet aus den Windmessungsdaten. Es wird eine (timesteps x region_split)-Matrix "wind_on_reg "in MW erstellt

#----------------------------
wind_on_env <- function(){
# weather_data einlesen, die Laendernummer wird aus der weather_station Tabelle zusaetzlich zugeordnet
sql_wind 		    <- paste("SELECT weather_data.station_id, weather_station.country_id, hour, wind_speed, wind_strong, weight_wind FROM weather_data, weather_station WHERE year = ",weather_year,"AND hour >=", start_timestep_weather,"AND hour <=",end_timestep_weather,"AND weather_data.station_id = weather_station.station_id", sep = " ")
wind_data 		  <- dbGetQuery(con_baltic, sql_wind)

# Stationsdaten einlesen zur Bestimmung der Gewichtung und ob Stark- oder Schwachwind
sql_station   	<- paste("SELECT station_id, country_id, wind_strong, weight_wind FROM weather_station", sep = " ")
station_data 		<- dbGetQuery(con_baltic, sql_station)

#----------------------------

# in Liste bringen fuer die Berechnung der Einspeise-Zeitreihen, eine Matrix pro Land, Stunden in den Reihen, Stationen in den Spalten

wind_list 	<- vector("list", 1 + region_abroad)
wind_on_reg <- matrix(0, nrow = end_timestep_weather, ncol = 1 + region_abroad)

korrektur <- c(1,1,1,1,1,.8,.85,.75,1) # fuer verschiedene rauhigkeiten um die messstationen

for(country in country_list){
  idx_country       <- which(country_list == country)
  weather_part 			<- subset(wind_data, country_id == country)
  weather_part      <- subset(weather_part, station_id <= (country+.5) ) # wegen hinzugefuegter station in lettland
  weather_part 			<- weather_part[order(weather_part$hour),]
  weather_part 			<- weather_part[order(weather_part$station_id),]
  stations 			    <- unique(weather_part$station_id)
  weather_matrix 		<- matrix(weather_part$wind_speed, ncol = length(stations), nrow = timesteps_weather)
  colnames(weather_matrix) 	    <- stations
  wind_list[[idx_country]] 		  <- weather_matrix
  
# Windgeschwindigkeit (in 10m Hoehe) ueber Leistungskennlinien zur Einspeiseleistung pro Station berechnen
  station_part      <- subset(station_data, country_id == country)
  station_part      <- station_part[order(station_part$station_id),]
  
  P_inst            <- as.numeric(as.character(scenario_re$wind_onshore[which(scenario_re$country_id==country)])) * 1000 # aus GW werden MW
  
  wind_on_matrix <- matrix(0, ncol=length(stations), nrow = timesteps_weather)
  
  for (station in 1:length(stations)){
    if(station_part$wind_strong[station]==1){
      
      # Starkwindleistungskennlinie: Enercon e-126 7580kW, Nabenhoehe 135m
      x<-c(2.9,3:17,39.9,40)
      y<-c(0,55,175,410,760,1250,1900,2700,3750,4850,5750,6500,7000,7350,7500,7580,7580,0)
      
      # Approximieren ueber Leistungskennlinie / Nennleistung * Gewichtung * installierter Leistung * Hoehenfaktor
      wind_on_matrix[,station] <- approx(x, y, xout=(weather_matrix[,station]*1.87*korrektur[country]), method='linear', rule = 2)$y / 7580 * station_part$weight_wind[station] * P_inst 
    
    }else{
      
      # Schwachwindleistungskennlinie: Repower 3.2M114 143m, Nabenhoehe 120m
      x<-c(2.9,3,4,5,6,7,8,9,10,11,12,21.99,22)
      y<-c(0,27,151,327,548,978,1471,2031,2594,3033,3200,3200,0)
      
      # Approximieren ueber Leistungskennlinie / Nennleistung * Gewichtung * installierter Leistung * Hoehenfaktor
      wind_on_matrix[,station] <- approx(x, y, xout=(weather_matrix[,station]*1.84*korrektur[country]), method='linear', rule = 2)$y / 3200 * station_part$weight_wind[station] * P_inst
    }
  }
  
  wind_on_reg[start_timestep_weather:end_timestep_weather,idx_country] <- rowSums(wind_on_matrix)  
  
}

#----------------------------
if (region_split_de > 1) {
  # Verteilung der Windeinspeisung auf die Regionen innerhalb Deutschlands
  wind_on_distribution        <- dbGetQuery(con_baltic, "SELECT capacity FROM ee_distribution WHERE technology = 'wind_onshore'")
  wind_on_distribution        <- as.numeric(as.character(unlist(wind_on_distribution)))
  wind_on_share               <- wind_on_distribution/sum(wind_on_distribution)
  wind_dpr_de                 <- matrix(0, nrow = timesteps_weather, ncol = 21)  
  wind_dpr_de[,1:18]          <- wind_on_reg[ ,1] %*% t(wind_on_share) 
  
  wind_reg_de <- matrix(NA, ncol = region_split_de, nrow = timesteps_weather)
  
  if (region_split_de < 21) {
    # die Aufteilung der dpr auf die Regionen ist in region_list gespeichert
    for(reg in 1:region_split_de){
      r <- region_list[[reg]]
      if(length(r) == 1){
        wind_reg_de[,reg] <- wind_dpr_de[,(region_list[[reg]]-100)]
      }else{
        wind_reg_de[,reg] <- rowSums(wind_dpr_de[,(region_list[[reg]]-100)])
      }
    }
  } else {
    wind_reg_de           <- wind_dpr_de
  }
  
  if(length(country_list)>1){
    wind_on_reg <- cbind(wind_reg_de, wind_on_reg[ , 2:length(country_list)])
  }else{
    wind_on_reg <- wind_reg_de
  }
}

#----------------------------
# Wenn die Rechnung fuer Viertelstunden gemacht wird, muessen die Daten interpoliert werden.
if (time_unit == "quarter") {
  wind_on_reg_quarter   <- matrix(0, ncol = region_split, nrow = end_timestep)
  idx_hour              <- seq.int(1,(end_timestep - 3), by = 4)
  diff_hour             <- diff(wind_on_reg)
  diff_hour             <- rbind(diff_hour, diff_hour[timesteps_weather-1,]) 
  # Die letzte Differenz wird zweimal verwendet
  
  wind_on_reg_quarter[idx_hour,] <- wind_on_reg
  
  wind_on_reg_quarter[idx_hour + 1, ]   <- wind_on_reg_quarter[idx_hour, ] + diff_hour/4
  wind_on_reg_quarter[idx_hour + 2, ]   <- wind_on_reg_quarter[idx_hour, ] + diff_hour/4*2
  wind_on_reg_quarter[idx_hour + 3, ]   <- wind_on_reg_quarter[idx_hour, ] + diff_hour/4*3
  
  wind_on_reg <- wind_on_reg_quarter
}
return(wind_on_reg)
}
wind_on_reg <- wind_on_env()

